import time
from utils import countdown
import pickle
import os.path
import numpy as np
import cv2

class GRDataset:
    def __init__(self, name, gestures, res=(128,128)):
        self.name = name
        self.gestures = gestures
        self.res = res

        print("Checking for existing dataset...")
        filename = "data/{}".format(self.name)
        if os.path.exists(filename):
            self.dataset = pickle.load(open(filename, "rb"))
            total = 0
            for v in self.dataset.values():
                total += len(v)
            print("Found existing dict with {} examples.".
                  format(total))
        else:
            print("No dataset found")
            self.dataset = {}


    def gather(self, N, CAM, gestureID):
        assert N > 0, print("You specified zero examples to be gathered.")

        print("Gathering gesture...")

        examples = []
        for k in range(N):
            img = CAM.render(self.res)
            examples.append(img)

        if gestureID in self.dataset:
            self.dataset[gestureID].extend(examples)
        else:
            self.dataset[gestureID] = examples

        print("Saving dataset")
        pickle.dump(self.dataset, open("data/{}".format(self.name), "wb"))


    def getBatch(self, N):
        """

        :param N: int, batchsize
        :return: [ndarray, ndarray], Returns list of N normalized float images with shape Nxhxw and cats with shape Nx7
        """
        ims = []
        cats = []

        n_categories = len(self.dataset)
        for i in range(N):
            cat = np.random.randint(0, n_categories)
            cat_ims = self.dataset[cat]
            img = cat_ims[np.random.randint(len(cat_ims))]
            img = cv2.normalize(img.astype('float32'), None, 0.0, 1.0, cv2.NORM_MINMAX)
            ims.append(img)
            cats.append(cat)

        ims = np.array(ims)
        cats = np.array(cats)

        return ims, cats


